/**
 * F10 资金流向功能实现
 *
 * 涉及到修改的地方有：
 *
 * ../defaults_s_dc.html 和 ../default_s_lc.html 需要引用该文件
 *
 * default.js line:287 处增加以下代码
    // F10-资金流向初始化
    CapitalDirection.init(obj);
 *
 * ../css/css/evening/objects.css 底部增加以下样式
    #capitalDirectionTable > table > thead > tr {
        background: transparent !important;
    }
    #capitalDirectionTable > table > tbody > tr > td:first-child {
        color: #fff !important;
    }
    #ring-container .highcharts-title {
        color: #fff !important;
        fill: #fff !important;
    }
    #ring-container .highcharts-data-labels.highcharts-tracker text > tspan:nth-child(3) {
        fill: #DDDDDD !important;
    }
    #outerCircle,
    #innerCircle {
        border: 10px solid rgba(51, 52, 56, 0.63) !important;
    }
    #ring-container .highcharts-series.highcharts-tracker path {
        stroke: rgba(47, 47, 47, 1);
    }
    #ring-container .highcharts-series.highcharts-tracker path:nth-child(4) {
        fill: #00EE00 !important;
    }
    #ring-container .highcharts-series.highcharts-tracker path:nth-child(3) {
        fill: #9BFF01 !important;
    }
 */
var CapitalDirection = new function() {
    /**
     * 初始化
     */
    this.init = function(id) {
        // APP 首次进入资金时 ID 为空字符串
        if (id === '' || id == '1') {
            // 防止重复添加节点
            if ($('#ring-container').length == 0) {
                $('<div id="ring-container" style="width:100%;height:250px;"></div>').appendTo($("#main"))
                $('<div id="outerCircle" style="height: 148px; width: 148px; position: absolute; border: 10px solid rgba(47, 47, 47, 0.06); border-radius: 50%; margin-top: -219px; margin-left: calc(50% - 84px); display: none;"></div>').appendTo("#main");
                $('<div id="innerCircle" style="height: 90px; width: 90px; position: absolute; border: 10px solid rgba(47, 47, 47, 0.06); border-radius: 50%; margin-top: -190px; margin-left: calc(50% - 55px); display: none;"></div>').appendTo("#main");
                $('<div id="capitalDirectionTable" style="width: 100%; height: 80px;"></div>').appendTo("#main");
                this.getCapitalDirection();
            }
        } else {
            this.removeAll();
        }
    }

    /**
     * 删除节点
     */
    this.removeAll = function() {
        if ($('#ring-container').length > 0) {
            $('#ring-container').remove();
            $('#outerCircle').remove();
            $('#innerCircle').remove();
            $('#capitalDirectionTable').remove();
        }
    }

    /**
     * 获取资金流向数据 [调用原生方法]
     */
    this.getCapitalDirection = function() {
        params = params || {}
        try {
            if (params.appType == "android") {
                //Android/iOS客户端类型判断
                KDS_Native.getCapitalDirection();
            } else if (params.appType == "ios") {
                this.loadURL("KDS_Native://getCapitalDirection");
            } else {
                // TODO 测试使用
                var json = {"BBD":"32966366","BBD10":"31236099","BBD5":"23433043","DDX":"515","DDX10":"-242","DDX5":"174","DDY":"0","DDY10":"290639444","DDY5":"279000000","DDZ":"110057","RatioBS":"1513","buyCount":"39597391","code":"300001","date":"20170811","dealCountMovingAverage":"35024470","fundsInflows":["18609649","17874960","9605074","9290025","7361126","15866393"],"fundsOutflows":["16153511","26642460","16560232","15854000","7474813","19538306"],"largeBuyAmount":"6550638","largeBuyDealCount":"0","largeBuyVolume":"421200","largeDiffer":"0","largeNetInflow":"836620","largeSellAmount":"5579978","largeSellDealCount":"0","largeSellVolume":"363562","mediumBuyAmount":"8469846","mediumBuyVolume":"545807","mediumDiffer":"0","mediumNetInflow":"-2079278","mediumSellAmount":"9669774","mediumSellVolume":"626000","othersFundsInflows":["1000406","2456138","-19944182","-18772434","42432","2654654"],"othersFundsOutflows":["4002034","422432","3232","432432","-10575","924492"],"sellCount":"27634132","smallBuyAmount":"8281240","smallBuyVolume":"535203","smallDiffer":"0","smallNetInflow":"-376860","smallSellAmount":"8287427","smallSellVolume":"535113","time":"13310000","ultraLargeBuyAmount":"3490465","ultraLargeBuyVolume":"224800","ultraLargeDiffer":"32568332","ultraLargeNetInflow":"1619518","ultraLargeSellAmount":"848613","ultraLargeSellVolume":"54300"};
                this.setCapitalDirection(JSON.stringify(json));
            }
        } catch (e) {
            Globals.Loading(false);
            $("body").append("app交互错误！");
        }
    }

    /**
     * 设置资金流向数据 [提供原生调用的方法]
     * @param {[type]} json [JSON 对象]
     */
    this.setCapitalDirection = function(json) {
        if ($('#ring-container').length > 0) {
            try {
                this.set(JSON.parse(json || '{}'));
            }catch (e){
                // 线上 android 返回的数据非正常 JSON 时，报错弹框会导致 APP 崩溃
                // alert(e);
            }
        }
    }

    this.set = function(data) {
        // 这里给那个列表填充数据
        /*
            主力流入 fundsInflows
            主力流出 fundsOutflows
            净流入 fundsInflows - fundsOutflows
            散户流入 othersFundsInflows
            散户流出 othersFundsOutflows
            净流出 othersFundsInflows - othersFundsOutflows
        */

        data.fundsInflows = data.fundsInflows || [];
        data.fundsOutflows = data.fundsOutflows || [];
        data.othersFundsInflows = data.othersFundsInflows || [];
        data.othersFundsOutflows = data.othersFundsOutflows || [];

        // TODO 测试无数据情况
        // data.fundsInflows = []
        // data.fundsOutflows = [];
        // data.othersFundsInflows = [];
        // data.othersFundsOutflows = [];

        var fundsInflows = this.isNumber(data.fundsInflows[0]) ? parseFloat(data.fundsInflows[0]) : 0,
            fundsOutflows = this.isNumber(data.fundsOutflows[0]) ? parseFloat(data.fundsOutflows[0]) : 0,
            othersFundsInflows = this.isNumber(data.othersFundsInflows[0]) ? parseFloat(data.othersFundsInflows[0]) : 0,
            othersFundsOutflows  = this.isNumber(data.othersFundsOutflows [0]) ? parseFloat(data.othersFundsOutflows [0]) : 0,
            total = ComputeUtils.accAdd(ComputeUtils.accAdd(fundsInflows, fundsOutflows),  ComputeUtils.accAdd(othersFundsInflows, othersFundsOutflows )) ;

        // selected: true 时，点击数据不会显示阴影
        var data = [];
        if (fundsInflows != 0) {
            data.push({name: '主力流入', y: ComputeUtils.floatDiv(fundsInflows, total), color:'#FF5900', selected: true});
        }
        if (othersFundsInflows != 0) {
            data.push({name: '散户流入', y: ComputeUtils.floatDiv(othersFundsInflows, total), color:'#FF9D00', selected: true});
        }
        if (othersFundsOutflows != 0) {
            data.push({name: '散户流出', y: ComputeUtils.floatDiv(othersFundsOutflows , total), color:'#20CC26', selected: true});
        }
        if (fundsOutflows != 0) {
            data.push({name: '主力流出', y: ComputeUtils.floatDiv(fundsOutflows, total), color:'#0DB14B', selected: true});
        }

        var pie_dataLabels_enabled = true;
        var pie_borderWidth = '1';

        // 资金流向没有数据时的处理
        if (fundsInflows == 0 && fundsOutflows == 0
            && othersFundsInflows == 0 && othersFundsOutflows == 0) {
            // data = [{
            //     name: '',
            //     y: 1,
            //     color:'#666666',
            //     selected: true
            // }];
            // pie_dataLabels_enabled = false;
            // pie_borderWidth = '0';
            // $('#outerCircle').hide();
            // $('#innerCircle').hide();

            this.removeAll();
        } else {
            $('#outerCircle').show();
            $('#innerCircle').show();
        }

        $('#ring-container').highcharts({
            chart: {
                backgroundColor: 'transparent',
                plotBackgroundColor: null,
                plotBorderWidth: null,
                plotShadow: false,
                spacing : [0, 0 , 20, 0]
            },
            title: {
                floating: true,
                // useHtml: true,
                style: {
                    color: '#000000',
                    fontSize: '16px',
                    fontFamily: 'PingFangSC-Medium'
                },
                text: '资金流向'
            },
            tooltip: {
                // 是否启用数据提示框
                enabled: false,
            },
            plotOptions: {
                pie: {
                    // 是否允许使用鼠标选中数据点
                    allowPointSelect: false,
                    cursor: 'pointer',
                    dataLabels: {
                        enabled: pie_dataLabels_enabled,
                        format: '<span style="font-family: PingFangSC-Medium; font-size: 24px; color: {point.color}; font-weight: bold;">{point.percentage:.1f}</span><span style="font-family: PingFangSC-Regular; font-size: 12px; color: {point.color}; line-height: 67px;">%</span> <br><span style="font-family: PingFangSC-Regular; font-size: 14px; color: #3D4454;">{point.name}</span>',
                        style: {
                            color: (Highcharts.theme && Highcharts.theme.contrastTextColor) || 'black',
                            fontSize: '14px',
                            fontFamily: 'PingFangSC-Medium'
                        }
                    },
                    borderWidth: pie_borderWidth,
                    borderColor: '#fff'
                },
                series: {
                    // 禁止加载时的动画
                    animation: false
                }
            },
            series: [{
                type: 'pie',
                dataLabels: {
                    inside: true,
                    distance: 10
                },
                innerSize: 110,
                size: 150,
                // name: '市场份额',
                data: data
            }]
        }, function(c) {
            // 环形图圆心
            var centerY = c.series[0].center[1],
                titleHeight = parseInt(c.title.styles.fontSize);
            c.setTitle({
                y: centerY + titleHeight/2
            });
            chart = c;
        });

        var tableHtml = this.setCapitalDirectionTableValue(this.getCapitalDirectionTable(), fundsInflows, fundsOutflows, othersFundsInflows, othersFundsOutflows );
        $('#capitalDirectionTable').html(tableHtml);
    }

    /**
     * 返回注释里的内容
     * @param function f [description]
     */
    this.HereDoc = function(f) {
        var doc = String(f).replace(/^[^\/]+\/\*!?\s?/, '').replace(/\*\/[^\/]+$/, '');
        return doc;
    }

    this.getCapitalDirectionTable = function() {
        return this.HereDoc(function() {/*
            <div id='capitalDirectionTable'>
                <table style='width: 90.8%; margin-left: 4.6%; text-align: center; font-family: PingFangSC-Regular; margin-bottom: 20px; margin-top: 5px;'>
                    <thead>
                        <tr style='font-size: 13px; color: #858C9E; height: 20px; background: #F8F9FB;'>
                            <td style='width: 19%'>分类</td>
                            <td style='width: 27%; text-align: left;'>流入(万元)</td>
                            <td style='width: 27%; text-align: left;'>流出(万元)</td>
                            <td style='width: 27%;'>净流入(万元)</td>
                        </tr>
                    </thead>
                    <tbody style='font-size: 15px;'>
                        <tr style='height: 30px;'>
                            <td style='color: #000000; padding-top: 10px;'>
                                主力
                            </td>
                            <td style='color: #FF5900; padding-top: 10px; text-align: left; font-family: PingFangSC-Medium;'>
                                <span style='font-size: 8px; position: relative; top: -2px;'>●</span>
                                {{zl_lr}}
                            </td>
                            <td style='color: #0DB14B; padding-top: 10px; text-align: left; font-family: PingFangSC-Medium;'>
                                <span style='font-size: 8px; position: relative; top: -2px;'>●</span>
                                {{zl_lc}}
                            </td>
                            <td style='padding-top: 10px; {{color_zl_zlr}} font-family: PingFangSC-Medium;'>
                                {{zl_zlr}}
                            </td>
                        </tr>
                        <tr style='height: 30px;'>
                            <td style='color: #000000;'>
                                散户
                            </td>
                            <td style='color: #FF8000; text-align: left; font-family: PingFangSC-Medium;'>
                                <span style='font-size: 8px; position: relative; top: -2px;'>●</span>
                                {{sh_lr}}
                            </td>
                            <td style='color: #26CA2D; text-align: left; font-family: PingFangSC-Medium;'>
                                <span style='font-size: 8px; position: relative; top: -2px;'>●</span>
                                {{sh_lc}}
                            </td>
                            <td style='{{color_sh_zlr}} font-family: PingFangSC-Medium;'>
                                {{sh_zlr}}
                            </td>
                        </tr>
                    </tbody>
                </table>
            </div>
        */});
    }

    this.setCapitalDirectionTableValue = function(tableHtml, fundsInflows, fundsOutflows, othersFundsInflows, othersFundsOutflows ) {
        var netInflow = ComputeUtils.subtr(fundsInflows, fundsOutflows);
        var netOutflow = ComputeUtils.subtr(othersFundsInflows, othersFundsOutflows );
        var color_zl_zlr = netInflow >= 0 ? 'color: #FF5900;' : 'color: #0DB14B;';
        var color_sh_zlr = netOutflow >= 0 ? 'color: #FF8000;' : 'color: #26CA2D;';
        return tableHtml
                .replace(/{{zl_lr}}/g, this.fommat(fundsInflows))
                .replace(/{{zl_lc}}/g, this.fommat(fundsOutflows))
                .replace(/{{zl_zlr}}/g, this.fommat(netInflow))

                .replace(/{{sh_lr}}/g, this.fommat(othersFundsInflows))
                .replace(/{{sh_lc}}/g, this.fommat(othersFundsOutflows ))
                .replace(/{{sh_zlr}}/g, this.fommat(netOutflow))

                .replace(/{{color_zl_zlr}}/g, color_zl_zlr)
                .replace(/{{color_sh_zlr}}/g, color_sh_zlr);
    }

    this.fommat = function(value) {
        return value === 0 || value === '0' ? '--' : parseFloat(ComputeUtils.floatDiv(value, 10000)).toFixed(2);
    }

    this.loadURL = function(url) {
        var iFrame;
        iFrame = document.createElement("iframe");
        iFrame.setAttribute("src", url);
        iFrame.setAttribute("style", "display:none;");
        iFrame.setAttribute("height", "0px");
        iFrame.setAttribute("width", "0px");
        iFrame.setAttribute("frameborder", "0");
        document.body.appendChild(iFrame);
        // 发起请求后这个iFrame就没用了，所以把它从dom上移除掉
        iFrame.parentNode.removeChild(iFrame);
        iFrame = null;
    }

    /**
     * 是否是纯数字
     * @param  {[type]}  value [description]
     * @return {Boolean}       [description]
     */
    this.isNumber = function(value) {
        return /^[0-9]+(.[0-9]*)?$/.test(value);
    }
};

setCapitalDirection = function(json) {
    CapitalDirection.setCapitalDirection(json);
}


var ComputeUtils = new function() {
    //加法函数
    this.accAdd = function(arg1, arg2) {
        var r1, r2, m;
        try {
            r1 = arg1.toString().split(".")[1].length;
        }
        catch (e) {
            r1 = 0;
        }
        try {
            r2 = arg2.toString().split(".")[1].length;
        }
        catch (e) {
            r2 = 0;
        }
        var m = Math.pow(10, Math.max(r1, r2));
        var n = (r1 >= r2) ? r1 : r2;
        return ((arg1 * m + arg2 * m) / m).toFixed(n);
    }

    //减法函数
    this.subtr = function (arg1, arg2) {
        var r1, r2, m, n;
        try {
            r1 = arg1.toString().split(".")[1].length;
        }
        catch (e) {
            r1 = 0;
        }
        try {
            r2 = arg2.toString().split(".")[1].length;
        }
        catch (e) {
            r2 = 0;
        }
        var m = Math.pow(10, Math.max(r1, r2));
        //last modify by deeka
        //动态控制精度长度
        var n = (r1 >= r2) ? r1 : r2;
        return ((arg1 * m - arg2 * m) / m).toFixed(n);
    }

    /**
     * 乘
     */
    this.floatMul = function(arg1, arg2) {
        var m = 0,
            s1 = arg1.toString(),
            s2 = arg2.toString();
        try {
            m += s1.split(".")[1].length
        } catch (e) {}
        try {
            m += s2.split(".")[1].length
        } catch (e) {}
        return Number(s1.replace(".", "")) * Number(s2.replace(".", "")) / Math.pow(10, m);
    }

    /**
     * 除
     */
    this.floatDiv = function(arg1, arg2) {
        var t1 = 0,
            t2 = 0,
            r1, r2;
        try {
            t1 = arg1.toString().split(".")[1].length
        } catch (e) {}
        try {
            t2 = arg2.toString().split(".")[1].length
        } catch (e) {}

        r1 = Number(arg1.toString().replace(".", ""));

        r2 = Number(arg2.toString().replace(".", ""));
        return (r1 / r2) * Math.pow(10, t2 - t1);
    }

    /**
     * 余
     */
    this.floatOver = function(arg1, arg2) {
        var r1, r2, m;
        try {
            r1 = arg1.toString().split(".")[1].length
        } catch (e) {
            r1 = 0
        }
        try {
            r2 = arg2.toString().split(".")[1].length
        } catch (e) {
            r2 = 0
        }
        var m = Math.pow(10, Math.max(r1, r2));
        return this.floatMul(arg1, m) % this.floatMul(arg2, m);
    }
};

