import 'whatwg-fetch';
import fetchJsonp from 'fetch-jsonp';
import KmcHostUrls from './kmc-hostUrls';

export default {

    getHostUrl() {
        if (KmcHostUrls.hostUrlType == 'test') {
            return KmcHostUrls.hostTestUrl;
        } else {
            return KmcHostUrls.hostPulicUrl;
        }
    },

    /**
     * get请求
     * @param {string} url 地址
     * @param {function} successCallBack 成功的回调
     */
    get(url, successCallBack) {
        url = this.getHostUrl() + url;
        fetch(url)
            .then(
                (response) => {
                    if (response.ok) {
                        return response.json()
                    } else {
                        console.error('服务器繁忙，请稍后再试；\r\nCode:' + response.status)
                    }
                }
            )
            .then(
                (result) => {
                    successCallBack && successCallBack(result);
                }
            )
            .catch(error => console.log('[error] ', error))
    },

    /**
     * post请求
     * @param {string} url 地址
     * @param {object} data 请求参数
     * @param {function} successCallBack 成功的回调
     */
    post(url, data, successCallBack) {
        url = this.getHostUrl() + url;
        fetch(url, {
            method: 'POST',
            headers: {
                'Content-type': 'application/json; charset=utf-8'
            },
            body: JSON.stringify(data)
        })
        .then(
            (response) => {
                if (response.ok) {
                    return response.json()
                } else {
                    console.error('服务器繁忙，请稍后再试；\r\nCode:' + response.status)
                }
            }
        )
        .then(
            (result) => {
                successCallBack && successCallBack(result);
            }
        )
        .catch(error => console.log('[error] ', error))
    },

    /**
     * jsonP请求
     * @param {string} url 地址
     * @param {function} successCallBack 成功的回调
     * @param {function} errorCallBack   错误的回调
     * @param {string} errorCallBack   错误的回调
     */
    jsonp(url, successCallBack, errorCallBack) {
        url = this.getHostUrl() + url;
        fetchJsonp(url, {
            timeout: 60 * 1000,
            jsonpCallback: 'callback'
        })
        .then(
            (response) => {
                if (response.ok) {
                    return response.json();
                } else {
                    console.error('服务器繁忙，请稍后再试；\r\nCode:' + response.status)
                }
            }
        ).then(
            (result) => {
                successCallBack && successCallBack(result);
            }
        )
        .catch(
            (error) => {
                errorCallBack && errorCallBack(error);
            }
        )
    }
}