import KmcLocalStorage from './kmc-localStorage';
import KmcConstants from './kmc-constants';
import KmcInteraction from './kmc-interaction';

export default {
    /**
     * 获取app的主题色
     */
    getAppTheme() {
        return document.documentElement.getAttribute('apptheme') || 'day';
    },

    /**
     * 获取URL中的参数
     */
    getUrlparam(param) {
        const reg = new RegExp('(^|&)' + param + '=([^&]*)(&|$)');
        const r = window.location.search.substr(1).match(reg) || window.location.hash.substring((window.location.hash.search(/\?/)) + 1).match(reg);
        if (r != null) {
            return decodeURIComponent(r[2]);
        }
        return null;
    },

    /**
     * 获取数组里的最大值
     */
    getArrMax(arr) {
        var max = arr[0];
        for (var i = 0; i < arr.length; i++) {
            if (max < arr[i]) {
                max = arr[i];
            }
        }
        return max;
    },

    /**
     * 获取原生传递过来的数据(old)
     * @param appVerion 客户端版本号 ( 用于后期版本客户端接口升级, Web判断做兼容提示等 )
     * @param appType   客户端类型 ( Web用于判断是调用App Android/iOS 哪端的方法 )
     * @param phoneNum  手机号
     * @param deviceId  Android设备号 或 iOS UUID
     * @param paramJson 扩展参数JSON格式( 股票代码等 )
     */
    callBackUserData(appVerion, appType, phoneNum, deviceID, paramsJson) {
        console.log(['callBackUserData', appVerion, appType, phoneNum, deviceID, paramsJson].join(', '));
        KmcLocalStorage.removeLocalData(KmcConstants.KDS_KEY.stockData);
        KmcLocalStorage.setLocalData(KmcConstants.KDS_KEY.stockData, paramsJson || {});
    },

    /**
     * 设置站点域名
     */
    setRequestHostUrls(hostUrls) {
        console.log(['callBackUserData', hostUrls].join(', '));
        if (hostUrls == "" || hostUrls == null || hostUrls == "undefined") {
            hostUrls = KmcLocalStorage.getLocalData(KmcConstants.KDS_KEY.hostUrls);
        } else {
            var urls = [];
            if (hostUrls.indexOf(",") > -1) {
                urls = hostUrls.split(",");
            } else {
                urls[0] = hostUrls;
            }

            var index = Math.floor((Math.random() * urls.length));
            var urltext = urls[index];
            if (urls[index].indexOf("?") > -1) {
                urltext = urls[index].split("?");
            }
            KmcLocalStorage.removeLocalData(KmcConstants.KDS_KEY.hostUrls);
            KmcLocalStorage.setLocalData(KmcConstants.KDS_KEY.hostUrls, urltext);
            //KmcInteraction.setDefaultHostUrl(urltext);
        }
    },

    /**
     * 设置字体大小
     */
    onFontSwitch(value) {
        console.log('onFontSwitch ', value);
    }
}