module.exports = {
    /**
     * 判断是数字
     * @param  {[Number|String]}    number [数字]
     * @return {Boolean}                   [数字: ture, 非数字: false]
     */
    isNumber: function(number) {
        return !isNaN(parseFloat(number)) && isFinite(number);
    },

    /**
     * 判断是数组
     * @param   {Object}    value   需要判断的对象
     * @return  {Boolean}   [数组: ture, 非数组: false]
     */
    isArrar: function(value) {
        return Object.prototype.toString.call(value) === '[object Array]';
    },

    /**
     * 检测输入内容是否为正整数
     * @param  {[type]} obj  [股票代码输入框对象]
     * @param  {[type]} type [数据类型]
     * @return {[type]}      [description]
     */
    checkIsPlus: function(obj, type) {
        type = type ? type: 'StkCode';
        var t = /^[0-9]*[1-9][0-9]*$/; //判断正整数
        return this.formCheck(t, obj, type, 2);
    },

    /**
     * 检测输入内容是否为数字字母组合
     * @param {[type]} obj  [股票代码输入框对象]
     * @param {[type]} type [数据类型]
     */
    checkIsNumChar: function(obj, type) {
        type = type ? type: 'StkCode';
        var t = /^[A-Z0-9a-z]{2}[0-9]*$/;
        return this.formCheck(t, obj, type, 2);
    },

    /**
     * 检测数量是否为正整数
     * @param {[type]} obj  [数量输入框对象]
     * @param {[type]} mmlb [买卖方向]
     * @param {[type]} zqdm [description]
     */
    checkVolume: function(obj, mmlb, zqdm) {
        mmlb = mmlb ? mmlb: 'B';
        zqdm = zqdm ? zqdm: '';
        // 已改为客户端不对委托数量进行限制
        var t = /^[0-9]*[1-9][0-9]*$/;
        return this.formCheck(t, obj, 'Volume', 2);
    },

    /**
     * 判断价格是否为正实数
     * @param {[type]} obj [价格输入框对象]
     */
    checkPrice: function(obj) {
        var t = /^\d+(\.\d+)?$/;
        // 对市价委托不进行价格校验
        if (obj.val() == '市价委托') {
            return true;
        }
        return this.formCheck(t, obj, 'Price', 2);
    },

    checkRQ: function(obj) {
        var t = /^[20]{2}[0-9-]*$/;
        return this.formCheck(t, obj, 'RQ', 2);
    },

    /**
     * 判断金额格式
     * @param {[type]} obj [金额]
     */
    checkMoney: function(obj) {
        var t = /^\d+(\.\d+)?$/;
        return t.test(obj);
    },

    /**
     * 自定义placeholder效果
     * @param {[type]} obj  [文本输入框对象]
     * @param {[type]} Type [description]
     */
    customPlaceholder: function(obj, Type) {
        var ywlx = document.body.getAttribute('data-ywlx');
        if (Type == 'StkCode') {
            $(obj).css({
                'width': '90%'
            });

            if (obj.val().length == 0) {
                if (ywlx == 'kfsjj') {
                    obj.attr({
                        'placeholder': '请输入基金代码'
                    });
                } else {
                    obj.attr({
                        'placeholder': '请输入证券代码'
                    });
                }
            } else {
                obj.val('');
                if (ywlx == 'kfsjj') {
                    obj.attr({
                        'placeholder': '输入基金代码有误'
                    });
                } else {
                    obj.attr({
                        'placeholder': '输入证券代码有误'
                    });
                }
            }
        } else if (Type == 'Price') {
            if (obj.val().length == 0) {

                if (document.title.trim() == '基金申购') {
                    obj.attr({
                        'placeholder': '请输入申购金额'
                    });
                } else if (document.title.trim() == '基金认购') {
                    obj.attr({
                        'placeholder': '请输入认购金额'
                    });
                } else if (document.title.trim() == '基金赎回') {
                    obj.attr({
                        'placeholder': '请输入赎回份额'
                    });
                } else if (document.title.trim() == '基金转换') {
                    obj.attr({
                        'placeholder': '请输入转换份额'
                    });
                } else if (document.title.trim() == '定投申请') {
                    obj.attr({
                        'placeholder': '请输入定投金额'
                    });
                } else if (document.title.trim() == '定赎申请') {
                    obj.attr({
                        'placeholder': '请输入定赎份额'
                    });
                } else {
                    obj.attr({
                        'placeholder': '请输入委托价格'
                    });
                }
            } else {
                obj.val('');
                if (document.title.trim() == '基金申购') {
                    obj.attr({
                        'placeholder': '请输入正确金额'
                    });
                } else if (document.title.trim() == '基金认购') {
                    obj.attr({
                        'placeholder': '请输入正确金额'
                    });
                } else if (document.title.trim() == '基金赎回') {
                    obj.attr({
                        'placeholder': '请输入正确份额'
                    });
                } else if (document.title.trim() == '基金转换') {
                    obj.attr({
                        'placeholder': '请输入正确的转换份额'
                    });
                } else if (document.title.trim() == '定投申请') {
                    obj.attr({
                        'placeholder': '输入金额有误'
                    });
                } else if (document.title.trim() == '定赎申请') {
                    obj.attr({
                        'placeholder': '输入份额有误'
                    });
                } else {
                    obj.attr({
                        'placeholder': '请输入正确的价格'
                    });
                }
            }
        } else if (Type == 'QMCS') {
            if (obj.val().length == 0) {
                obj.attr({
                    'placeholder': '请输入期满次数'
                });
            } else {
                obj.attr({
                    'placeholder': '输入期满次数有误'
                });
                obj.val('');
            }
        } else if (Type == 'QMJE') {
            if (obj.val().length == 0) {
                obj.attr({
                    'placeholder': '请输入期满金额'
                });
            } else {
                obj.attr({
                    'placeholder': '输入期满金额有误'
                });
                obj.val('');
            }
        } else if (Type == 'Amount') {
            if (obj.val().length == 0) {
                obj.attr({
                    'placeholder': '请输入还款金额'
                });
            } else {
                obj.attr({
                    'placeholder': '请输入正确的还款金额'
                });
                obj.val('');
            }
        } else if (Type == 'hqVolume') {
            if (obj.val().length == 0) {
                obj.attr({
                    'placeholder': '请输入还券数量'
                });
            } else {
                obj.attr({
                    'placeholder': '还券数量输入有误'
                });
                obj.val('');
            }
        } else if (Type == 'Volume') {
            if (obj.val().length == 0) {
                obj.attr({
                    'placeholder': '请输入买卖数量'
                });
            } else {
                obj.attr({
                    'placeholder': '输入买卖数量有误'
                });
                obj.val('');
            }
        } else if (Type == 'RQ') {
            if (obj.val().length == 0) {
                obj.attr({
                    'placeholder': '请选择日期'
                });
            } else {
                obj.attr({
                    'placeholder': '输入日期有误'
                });
                obj.val('');
            }
        }
        obj.focus();
    },

    /**
     * 表单检查
     * @param {[type]} t           [正则表达式]
     * @param {[type]} obj         [输入框对象]
     * @param {[type]} Type        [检查类型]
     * @param {[type]} parentLevel [父Div层级]
     */
    formCheck: function(t, obj, Type, parentLevel) {
        var parentDiv;

        if (parentLevel == 2) {
            parentDiv = obj.parent().parent();
        } else if (parentLevel == 3) {
            parentDiv = obj.parent().parent().parent();
        }

        // 正则校验不通过
        if (!t.test(obj.val())) {
            // 设置错误状态提示语
            this.customPlaceholder(obj, Type);
            // 在父div上添加告警样式
            parentDiv.addClass('has-error');
            // 将焦点转移到出错的div上
            parentDiv.focus();
            return false;
        } else {
            if (parentDiv.hasClass('has-error')) {
                parentDiv.removeClass('has-error')
            }
            return true;
        }
    },

    /**
     * 删除数组中 undefined
     * @param  {[type]} array [数组]
     * @return {[type]}       [description]
     */
    deleteArrayUndefined: function(array) {
        var b = []; //去除undefined后的结果
        var len = array.length;
        for (var i = 0; i < len; i++) {
            if (typeof(array[i]) != 'undefined') {
                b.push(array[i]);
            }
        }
        return b;
    },

    /**
     * 为数字加上单位：万或亿
     *
     * 例如：
     *      1000.01 => 1000.01
     *      10000 => 1万
     *      99000 => 9.9万
     *      566000 => 56.6万
     *      5660000 => 566万
     *      44440000 => 4444万
     *      11111000 => 1111.1万
     *      444400000 => 4.44亿
     *      40000000,00000000,00000000 => 4000万亿亿
     *      4,00000000,00000000,00000000 => 4亿亿亿
     *
     * @param {number} number 输入数字.
     * @param {number} decimalDigit 小数点后最多位数，默认为2
     * @return {string} 加上单位后的数字
     */
    addChineseUnit: function() {
        var addWan = function(integer, number, mutiple, decimalDigit) {
            var digit = getDigit(integer);
            if (digit > 3) {
                var remainder = digit % 8;
                if (remainder >= 5) { // ‘十万’、‘百万’、‘千万’显示为‘万’
                    remainder = 4;
                }
                return Math.round(number / Math.pow(10, remainder + mutiple - decimalDigit)) / Math.pow(10, decimalDigit) + '万';
            } else {
                return Math.round(number / Math.pow(10, mutiple - decimalDigit)) / Math.pow(10, decimalDigit);
            }
        };

        var getDigit = function(integer) {
            var digit = -1;
            while (integer >= 1) {
                digit++;
                integer = integer / 10;
            }
            return digit;
        };

        return function(number, decimalDigit) {
            decimalDigit = decimalDigit == null ? 2 : decimalDigit;
            var integer = Math.floor(number);
            var digit = getDigit(integer);
            // ['个', '十', '百', '千', '万', '十万', '百万', '千万'];
            var unit = [];
            if (digit > 3) {
                var multiple = Math.floor(digit / 8);
                if (multiple >= 1) {
                    var tmp = Math.round(integer / Math.pow(10, 8 * multiple));
                    unit.push(addWan(tmp, number, 8 * multiple, decimalDigit));
                    for (var i = 0; i < multiple; i++) {
                        unit.push('亿');
                    }
                    return unit.join('');
                } else {
                    return addWan(integer, number, 0, decimalDigit);
                }
            } else {
                return number;
            }
        };
    }(),

    /**
     * 获取数字单位
     * @param  {[type]} value [description]
     * @return {[type]}       [description]
     */
    getChineseUnit: function(value) {
        if (value == null) {
            return '';
        }

        try {
            value = typeof(value) !== 'string' ? value.toString() : value;
            // 取小数点前的金额
            value = value.split('.')[0] || '0';
            // 通过转换数据解决数字前带 0 的问题
            value = parseFloat(value);
            value = value.toString();
        } catch (e) {
            value = '';
        }

        var unit = '';
        switch(value.length) {
            case 5:
                unit = '万';
                break;
            case 6:
                unit = '十万';
                break;
            case 7:
                unit = '百万';
                break;
            case 8:
                unit = '千万';
                break;
            case 9:
                unit = '亿';
                break;
            case 10:
                unit = '十亿';
                break;
            case 11:
                unit = '百亿';
                break;
            case 12:
                unit = '千亿';
                break;
        }
        return unit;
    },

    /**
    * 格式化金额 - 千分位
    * @param  {[String|Number]} money      [金额]                [NOT NULL]
    * @param  {[String|Number]} fixedPoint [保留小数位数]        [NULL]
    * @return {[Stirng]}                   [千分位格式化的金额]
    */
    formatMoney: function(money, fixedPoint) {
        if (money == null || isNaN(money) || money === "") {
            return "";
        }
        money = /^[0-9]+$/.test(fixedPoint)
            ? (parseFloat(money).toFixed(fixedPoint).toString())
            : money;
        var regexp = /^[0-9]+$/.test(money)
            ? /\d{1,3}(?=(\d{3})+?$)/g
            : /\d{1,3}(?=(\d{3})+?(\.\d*)$)/g;
        return money.toString().replace(regexp, '$&,');
    }
}
