/**
 * 适当改造 - 开放式基金交易
 *
 * @description
 * 如果使用该模块，必须在可买卖数量查询返回结果里调用
 *     PROPER_RECONSTRUCTION_OPEN_END_FUND_TRADING.init(response);
 *
 * 使用下面方法，获取是否打开适当改造功能模块
 *     PROPER_RECONSTRUCTION_OPEN_END_FUND_TRADING.getFunctionOpen();
 *
 * 申购、认购、转换、定投，在弹框上点击 “确定” 后调用 （向服务器发起委托下单前触发）
 *     PROPER_RECONSTRUCTION_OPEN_END_FUND_TRADING.start();
 *
 * 重置数据
 *     PROPER_RECONSTRUCTION_OPEN_END_FUND_TRADING.reset();
 *
 * 是否需要 CA (专业投资者)
 *     PROPER_RECONSTRUCTION_OPEN_END_FUND_TRADING.isNeedCA();
 *
 * 签署 CA (专业投资者)
 *     PROPER_RECONSTRUCTION_OPEN_END_FUND_TRADING.signCA();
 *
 * 获取业务流水号 (应周老板要求，有业务流水号时，所有下单都需要带上)
 *     PROPER_RECONSTRUCTION_OPEN_END_FUND_TRADING.getYwlsh();
 *
 */
var PROPER_RECONSTRUCTION_OPEN_END_FUND_TRADING = function () {
    // 开放式基金 MD5 在本地存储中的 KEY 值
    var OPEN_END_FUND_TRADING_MD5_KEY = 'OPEN_END_FUND_TRADING_MD5';
    // 开放式基金警示函内容的在本地存储中的 KEY 值
    var OPEN_END_FUND_TRADING_WARNING_LETTER_KEY = 'OPEN_END_FUND_TRADING_WARNING_LETTER';
    // 开放式基金确认书 MD5 在本地存储中的 KEY 值
    var OPEN_END_FUND_TRADING_MD5_CONFIRMATION_KEY = 'OPEN_END_FUND_TRADING_MD5_CONFIRMATION';
    // 开放式基金确认书内容的在本地存储中的 KEY 值
    var OPEN_END_FUND_TRADING_CONFIRMATION_KEY = 'OPEN_END_FUND_TRADING_CONFIRMATION';
    // 标题名称列表 (只有在这个列表下的功能才做适当性改造)
    var TITLE_NAME_ARRAY = ['基金申购', '基金认购', '基金转换', '定投申购'];
    // 填充模板key
    var CNSDX_TCMBZ = '{{name}}:{{account}}:{{service}}:{{risklevel}}:{{myrisklevel}}:{{investtime}}:{{myinvesttime}}:{{investtype}}:{{myinvesttype}}:{{signdate}}:{{riskMatch}}:{{investtimeMatch}}:{{investtypeMatch}}';

    var defaultState = function defaultState() {
        return {
            // 功能开关
            functionOpen: false,
            // 允许下单
            allowOrders: false,
            // CA 校验
            isCA: false,
            // 是风险匹配的
            isRiskMatching: false,
            // 是否专业投资者
            isProfessionalInvestors: false,
            // 成功走完流程后，执行的方法
            successHandleMethod: null,
            // 模板需要的字段
            templateField: {},
            // 基金信息
            jjxx: {},
            // 影像 (相关的信息保存在这里面，方便传值)
            yx: {
                // 保存入参信息
                'input': {},
                // 保存出参信息
                'output': {}
            },
            // 挑战码 (相关的信息保存在这里面，方便传值)
            tzm: {}
        };
    };

    var state = defaultState();

    /**
     * 初始化方法
     */
    function init(response) {
        var jjxx = response.cxlb[0];
        state.jjxx = jjxx;
        state.functionOpen = jjxx.xbbz === '1' ? true : false;
        state.isRiskMatching = jjxx.fxpp === '1' ? true : false;
        state.allowOrders = jjxx.yxxd === '0' ? true : false;
        state.isCA = jjxx.cabz === '1' ? true : false;
        state.isProfessionalInvestors = jjxx.tzzlx === '1' ? true : false;

        initTemplateField();

        state.tzm = {};
    }

    /**
     * 初始化模板需要的字段
     */
    function initTemplateField() {
        // 用户登录信息
        var dlxx = null;
        try {
            dlxx = JSON.parse(getLocalData('ActivePTJYUser')).body.dlxx[0];
        } catch (e) {
            showMessageTips('用户登录信息已失效，请您退出交易，重新进行交易登录！');
            return '';
        }

        // 姓名/名称
        var name = dlxx.khxm;
        // 资金账号
        var account = dlxx.zjzh;
        // 金融服务
        var service = state.jjxx.jjmc;
        // 风险等级（基金风险等级）
        var risklevel = state.jjxx.jjfxdjsm || '';
        // 风险承受能力等级（用户风险等级）
        var myrisklevel = state.jjxx.khfxdjsm || '';
        // 投资期限
        var investtime = state.jjxx.tzqxbzsm || '';
        // 预计投资期限
        var myinvesttime = state.jjxx.khtzqxbzsm || '';
        // 所属的投资品种
        var investtype = state.jjxx.tzlbbzsm || '';
        // 确认的投资品种
        var myinvesttype = state.jjxx.khtzlbbzsm || '';
        // 日  期
        var signdate = new Date().Format('yyyy年MM月dd日');

        // 风险等级匹配结果
        var riskMatch = state.jjxx.fxdjppjg || '';
        // 投资期限匹配结果
        var investtimeMatch = state.jjxx.tzqxppjg || '';
        // 投资品种匹配结果
        var investtypeMatch = state.jjxx.tzlbppjg || '';

        state.templateField = {
            'name': name,
            'account': account,
            'service': service,
            'risklevel': risklevel,
            'myrisklevel': myrisklevel,
            'investtime': investtime,
            'myinvesttime': myinvesttime,
            'investtype': investtype,
            'myinvesttype': myinvesttype,
            'signdate': signdate,
            'riskMatch': riskMatch,
            'investtimeMatch': investtimeMatch,
            'investtypeMatch': investtypeMatch
        };
    }

    /**
     * 获取适当性 - 开放式基金的开关状态
     * @return {[boolean]} [true: 打开, false: 关闭]
     */
    function getFunctionOpen() {
        // 专业投资者不需要适当性匹配 update 2017.10.30
        return state.functionOpen && !state.isProfessionalInvestors;
    }

    /**
     * 是否存在该标题名称
     * @param  {[String]} name [标题名称]
     * @return {[boolean]} [true: 存在, false: 不存在]
     */
    function isExistTileName(titleName) {
        return TITLE_NAME_ARRAY.indexOf(titleName) > -1;
    }

    /**
     * 开始流程
     */
    function start(successHandleMethod) {
        state.successHandleMethod = successHandleMethod;

        // 风险不匹配 && 不允许下单
        if (!state.isRiskMatching && !state.allowOrders) {
            submitError();
        }
        // 风险不匹配 && 允许下单
        else if (!state.isRiskMatching && state.allowOrders) {
                getInappropriateWarningLetter();
            }
            // 风险匹配
            else if (state.isRiskMatching) {
                    getAppropriateConfirmation();
                }
    }

    /**
     * 提交失败 (风险不匹配 && 不允许下单)
     */
    function submitError() {
        var modal = {
            'title': '委托失败',
            'msg': [{ class: 'align-left', 'value': '尊敬的客户：由于您的风险测试能力与产品风险等级不匹配，我们对您实施了投资特别保护，暂不允许委托此种证券产品。如有问题可咨询开户营业部或我公司全国统一资讯热线95587。' }]
        };
        var options = {
            'isController': true,
            'wrongIcoHide': true,
            'firstBtnHide': true,
            'rightButtonClick': function rightButtonClick() {
                Dialog.closeAlert('modalTips');
            }
        };
        Dialog.alert(controller.scope, controller.compile, modal, options);
    }

    /**
     * 获取模板的参数
     *
     * @description
     * 获取警示函和确认书的影像时都需要将该参数传过去
     */
    function getTemplateParameter() {
        return [state.templateField.name, state.templateField.account, state.templateField.service, state.templateField.risklevel, state.templateField.myrisklevel, state.templateField.investtime, state.templateField.myinvesttime, state.templateField.investtype, state.templateField.myinvesttype, state.templateField.signdate, state.templateField.riskMatch, state.templateField.investtimeMatch, state.templateField.investtypeMatch].join(":");
    }

    /**
     * 获取不适当警示函
     */
    function getInappropriateWarningLetter() {
        var md5 = getLocalData(OPEN_END_FUND_TRADING_MD5_KEY) || '';
        // 客户属性（'0'：个人  '1'：机构）
        var khsx = '';
        try {
            khsx = JSON.parse(getLocalData("ActivePTJYUser"))["body"]["dlxx"][0].khsx;
        } catch (error) {
            showMessageTips('用户登录信息已失效，请您退出交易，重新进行交易登录！');
            return;
        }
        // 影像类别 ['1': '个人场内不适当确认书', '2': '个人开放式基金适当确认书', '3': '个人开放式基金不适当确认书', '6': '机构开放式基金适当确认书', '7': '机构开放式基金不适当确认书']
        var yxlb = khsx === '0' ? '3' : khsx === '1' ? '7' : '';
        var postData = {
            // 适当性业务类型 ['0': '场内基金', '1': '开放式基金']
            'ywlx': '1',
            'yxlb': yxlb,
            'md5': md5,
            'khsx': khsx

            // 保存影像的入参，方便后面流程使用
        };state.yx.input = postData;

        ajaxPost(getApiUrl() + "ptjy/yxxt/hqyx", postData, function (response) {
            var fhxx = (response.fhxx || [])[0] || {};
            var content = fhxx.xy || '';
            var md5Value = fhxx.md5 || '';

            state.yx.input.tcmbk = CNSDX_TCMBZ;
            state.yx.input.tcmbz = getTemplateParameter();
            state.yx.output = fhxx;

            if (md5 === md5Value) {
                content = getLocalData(OPEN_END_FUND_TRADING_WARNING_LETTER_KEY) || '';
            } else {
                setLocalData(OPEN_END_FUND_TRADING_MD5_KEY, md5Value);
                setLocalData(OPEN_END_FUND_TRADING_WARNING_LETTER_KEY, content);
            }

            content = contentReplace(content);

            inappropriateWarningLetterAlert(content);
        });
    }

    /**
     * 获取适当确认书
     */
    function getAppropriateConfirmation() {
        var md5 = getLocalData(OPEN_END_FUND_TRADING_MD5_CONFIRMATION_KEY) || '';
        // 客户属性（'0'：个人  '1'：机构）
        var khsx = '';
        try {
            khsx = JSON.parse(getLocalData("ActivePTJYUser"))["body"]["dlxx"][0].khsx;
        } catch (error) {
            showMessageTips('用户登录信息已失效，请您退出交易，重新进行交易登录！');
            return;
        }
        // 影像类别 ['1': '个人场内不适当确认书', '2': '个人开放式基金适当确认书', '3': '个人开放式基金不适当确认书', '6': '机构开放式基金适当确认书', '7': '机构开放式基金不适当确认书']
        var yxlb = khsx === '0' ? '2' : khsx === '1' ? '6' : '';

        var postData = {
            // 适当性业务类型 ['0': '场内基金', '1': '开放式基金']
            'ywlx': '1',
            'yxlb': yxlb,
            'md5': md5,
            'khsx': khsx
            // 保存影像的入参，方便后面流程使用
        };state.yx.input = postData;

        ajaxPost(getApiUrl() + "ptjy/yxxt/hqyx", postData, function (response) {
            var fhxx = (response.fhxx || [])[0] || {};
            var content = fhxx.xy || '';
            var md5Value = fhxx.md5 || '';

            state.yx.input.tcmbk = CNSDX_TCMBZ;
            state.yx.input.tcmbz = getTemplateParameter();
            state.yx.output = fhxx;

            if (md5 === md5Value) {
                content = getLocalData(OPEN_END_FUND_TRADING_CONFIRMATION_KEY) || '';
            } else {
                setLocalData(OPEN_END_FUND_TRADING_MD5_CONFIRMATION_KEY, md5Value);
                setLocalData(OPEN_END_FUND_TRADING_CONFIRMATION_KEY, content);
            }

            content = contentReplace(content);

            properContentAlert(content);
        });
    }

    /**
     * 内容替换
     */
    function contentReplace(content) {
        return content.replace(/{{name}}/g, state.templateField.name).replace(/{{account}}/g, state.templateField.account).replace(/{{service}}/g, state.templateField.service).replace(/{{risklevel}}/g, state.templateField.risklevel).replace(/{{myrisklevel}}/g, state.templateField.myrisklevel).replace(/{{investtime}}/g, state.templateField.investtime).replace(/{{myinvesttime}}/g, state.templateField.myinvesttime).replace(/{{investtype}}/g, state.templateField.investtype).replace(/{{myinvesttype}}/g, state.templateField.myinvesttype).replace(/{{signdate}}/g, state.templateField.signdate).replace(/{{riskMatch}}/g, state.templateField.riskMatch).replace(/{{investtimeMatch}}/g, state.templateField.investtimeMatch).replace(/{{investtypeMatch}}/g, state.templateField.investtypeMatch);
    }

    /**
     * 不适当警示函弹出框
     */
    function inappropriateWarningLetterAlert(content) {
        var isTime = false;
        var fixedTime = 10;
        var butText = '签署({{s}}s)';
        var time = fixedTime;
        var interval = null;
        var isScrollBottom = false;
        var isIntervalDone = false;
        var intervalFun = function intervalFun() {
            if (time > 0) {
                controller.scope.modal.rightButtonText = butText.replace('{{s}}', time--);
            } else {
                clearInterval(interval);
                time = fixedTime;
                controller.scope.modal.rightButtonText = '签署';

                isIntervalDone = true;
                // 强制 10S 阅读且下拉至底部时才能点击 `签署` 按钮
                if (isScrollBottom) {
                    controller.scope.modal.isRightButtonDisabled = false;
                }
            }

            controller.scope.$digest();
        };
        interval = setInterval(intervalFun, 1000);

        var modal = {
            'title': '签署警示函',
            'isHtmlContent': true,
            'htmlContent': content
        };
        var options = {
            'isController': true,
            'leftButtonText': '拒绝',
            'isRightButtonDisabled': true,
            'rightButtonText': butText.replace('{{s}}', time--),
            'footerShow': false,
            'htmlContentFooterShow': true,
            leftButtonClick: function leftButtonClick() {
                Dialog.closeAlert('modalTips');
                onAgentEvent('普通交易>适当性改造>场外交易', '签署警示函>拒绝');
                clearInterval(interval);
            },
            rightButtonClick: function rightButtonClick() {
                Dialog.closeAlert('modalTips');
                onAgentEvent('普通交易>适当性改造>场外交易', '签署警示函>签署');
                saveContent(content);
            },
            onScrollBottom: function onScrollBottom(e) {
                isScrollBottom = true;
                // 强制 10S 阅读且下拉至底部时才能点击 `签署` 按钮
                if (isIntervalDone) {
                    controller.scope.modal.isRightButtonDisabled = false;
                    controller.scope.$digest();
                }
                console.log('滚动条已滚动到底部！');
            }

        };
        Dialog.alert(controller.scope, controller.compile, modal, options);
    }

    /**
     * 适当内容的弹框
     *
     * @description
     * 包含不适当警示函、适当确认书的内容展示
     */
    function properContentAlert(content) {
        var modal = {
            'title': '提示',
            'isHtmlContent': true,
            'htmlContent': content
        };
        var options = {
            'isController': true,
            'leftButtonText': '拒绝',
            'rightButtonText': '同意',
            'isRightButtonDisabled': true,
            'footerShow': false,
            'htmlContentFooterShow': true,
            leftButtonClick: function leftButtonClick() {
                Dialog.closeAlert('modalTips');
                onAgentEvent('普通交易>适当性改造>场外交易', '适当确认书>拒绝');
            },
            'rightButtonClick': function rightButtonClick() {
                Dialog.closeAlert('modalTips');
                onAgentEvent('普通交易>适当性改造>场外交易', '适当确认书>同意');
                saveContent(content);
            },
            onScrollBottom: function onScrollBottom(e) {
                controller.scope.modal.isRightButtonDisabled = false;
                controller.scope.$digest();
            }
        };
        Dialog.alert(controller.scope, controller.compile, modal, options);
    }

    /**
     * 保存内容
     *
     * @description
     * 提交到服务器保存影像内容
     */
    function saveContent(content) {
        var postData = {
            'ywlx': state.yx.input.ywlx,
            'yxlb': state.yx.input.yxlb,
            'khsx': state.yx.input.khsx,
            'tcmbk': CNSDX_TCMBZ,
            'tcmbz': state.yx.input.tcmbz,
            'ywlsh': state.yx.output.ywlsh,
            'cpdm': state.jjxx.jjdm,
            'cabz': state.jjxx.cabz,
            'dzhtjybs': state.jjxx.dzhtjybs
        };
        ajaxPost(getApiUrl() + 'ptjy/yxxt/cwbcyx', postData, function (response) {
            // 需要调用 CA
            if (state.isCA) {
                // 是否存在缓存ID
                var isCache = state.tzm.hcid != null ? true : false;
                // 显示短信验证码弹框
                Dialog.SMSAuthenticationCodeAlert(null, null, null, getChallengeCode, checkChallengeCode, isCache);
            } else {
                end();
            }
        }, function (response) {
            var message = response.cljg[0].message;
            Dialog.errorAlert(message, '提示');
        });
    }

    /**
     * 获取挑战码
     */
    function getChallengeCode(sjhm, handlerMethod) {
        var tradeUrl = 'ptjy/yxxt/hqtzm';
        var postData = {
            'yxlb': state.yx.input.yxlb,
            'ywlx': state.yx.input.ywlx,
            'sjhm': sjhm,
            'khsx': state.yx.input.khsx,
            'jjdm': state.jjxx.jjdm,
            'ywlsh': state.yx.output.ywlsh
        };

        ajaxPost(getApiUrl() + tradeUrl, postData, function (response) {
            var fhxx = {};
            try {
                fhxx = response.fhxx[0];
            } catch (error) {
                showMessageTips('服务器返回的数据格式错误！');
                return;
            }
            state.tzm = {
                'sjhm': sjhm,
                'hcid': fhxx.hcid || ''
            };

            if (fhxx.tzm) {
                handlerMethod(fhxx.tzm, state.tzm.hcid);
            } else {
                handlerMethod(null, state.tzm.hcid);
            }
        });
    }

    /**
     * 验证挑战码
     */
    function checkChallengeCode(code) {
        var tradeUrl = 'ptjy/yxxt/yztzm';
        var postData = {
            'tzm': code,
            'hcid': state.tzm.hcid || '',
            'sjhm': state.tzm.sjhm || '',
            'yxlb': state.yx.input.yxlb,
            'ywlx': state.yx.input.ywlx,
            'ywlsh': state.yx.output.ywlsh,
            'jjdm': state.jjxx.jjdm,
            'dzhtjybs': state.jjxx.dzhtjybs
        };
        ajaxPost(getApiUrl() + tradeUrl, postData, function (response) {
            end();
        });
    }

    /**
     * 结束流程
     */
    function end() {
        // 调用委托下单方法
        state.successHandleMethod(state.yx.output.ywlsh);
    }

    /**
     * 重置数据
     * @return {[type]} [description]
     */
    function reset() {
        state = defaultState();
    }

    /**
     * 是否需要 CA (专业投资者)
     */
    function isNeedCA() {
        return state.isProfessionalInvestors && state.isCA;
    }

    /**
     * 签署 CA (专业投资者)
     * @return {[type]} [description]
     */
    function signCA(successHandleMethod) {
        state.successHandleMethod = successHandleMethod;

        // 客户属性（'0'：个人  '1'：机构）
        var khsx = '';
        try {
            khsx = JSON.parse(getLocalData("ActivePTJYUser"))["body"]["dlxx"][0].khsx;
        } catch (error) {
            showMessageTips('用户登录信息已失效，请您退出交易，重新进行交易登录！');
            return;
        }

        // 获取挑战码、校验挑战码时需要用到的入参
        state.yx.input = {
            'ywlx': '1',
            // 产品合同
            'yxlb': '11',
            'khsx': khsx
        };
        state.yx.output.ywlsh = SMJJ.getState().output.ywlsh;

        // 是否存在缓存ID
        var isCache = state.tzm.hcid != null ? true : false;
        // 显示短信验证码弹框
        Dialog.SMSAuthenticationCodeAlert(null, null, null, getChallengeCode, checkChallengeCode, isCache);
    }

    function getYwlsh() {
        return state.yx.output.ywlsh || '';
    }

    return {
        init: init,
        getFunctionOpen: getFunctionOpen,
        isExistTileName: isExistTileName,
        start: start,
        reset: reset,
        isNeedCA: isNeedCA,
        getYwlsh: getYwlsh,
        signCA: signCA
    };
}();