/**
 * 私募基金
 *
 * 申购、认购、转换、定投，在可买卖数量查询时同步获取合同链接
 *     SMJJ.start('000001')
 *
 * 重置模块数据
 *     SMJJ.reset()
 *
 * 获取 state
 *     SMJJ.getState()
 *
 */
var SMJJ = function () {
    var state = defaultState();

    function defaultState() {
        return {
            // 请求成功
            'requestSuccess': false,
            // 产品说明书链接
            'cpsmslj': '',
            // 产品合同书链接
            'cphtslj': '',
            // 产品风险揭示书链接
            'cpfxjsslj': '',
            // 是否显示三个电子协议
            'show': false,
            // 获取三个电子协议的出参
            'output': {}
        };
    }

    function getState() {
        return state;
    }

    function setState(object) {
        state = Object.assign(state, object);
    }

    /**
     * 开始私募基金获取三个协议
     * @param  {[type]} jjdm     [基金代码]
     * @param  {[type]} dzhtjybs [电子合同签约标识] [0: 代表不需要签约, 1: 代表需要签约电子合同]
     */
    function start(jjdm, dzhtjybs) {
        reset();
        if (dzhtjybs == '1') {
            queryHqhtlj(jjdm);
            setState({ show: true });
        }
    }

    function concatS(ljqz, lj) {
        if (ljqz == null || ljqz == '' || lj == null || lj == '') {
            return '';
        }
        return ljqz + lj;
    }

    function queryHqhtlj(jjdm) {
        var sqlsh = PROPER_RECONSTRUCTION_OPEN_END_FUND_TRADING.isNeedCA() ? '1' : '';
        var postData = {
            jjdm: jjdm,
            sqlsh: sqlsh
        };
        ajaxPost(getApiUrl() + 'ptjy/yxxt/hqhtlj', postData, function (response) {
            var fhxx = response.fhxx[0];
            var cpsmslj = concatS(fhxx.ljqz, fhxx.cpsmslj);
            var cphtslj = concatS(fhxx.ljqz, fhxx.cphtslj);
            var cpfxjsslj = concatS(fhxx.ljqz, fhxx.cpfxjsslj);

            setState({
                requestSuccess: true,
                cpsmslj: cpsmslj,
                cphtslj: cphtslj,
                cpfxjsslj: cpfxjsslj,
                output: fhxx
            });
        });
    }

    /**
     * 重置数据
     */
    function reset() {
        state = defaultState();
    }

    function errorAlert() {
        var modal = {
            'title': '提示信息',
            'msg': [{ 'value': '网络异常，获取产品协议失败，请重新输入基金代码！' }]
        };
        var options = {
            'isController': true,
            'wrongIcoHide': true,
            'firstBtnHide': true,
            'rightButtonClick': function rightButtonClick() {
                Dialog.closeAlert('modalTips');
            }
        };
        Dialog.alert(controller.scope, controller.compile, modal, options);
    }

    return {
        getState: getState,
        start: start,
        reset: reset,
        errorAlert: errorAlert
    };
}();