var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

/**
 * APP 调用 H5 提供的方法
 *
 * 1. 以后与 APP 交互只需要约定 KEY 值，H5 根据原生传入的 KEY 来处理对应的逻辑，回调原生方法 (callbackTradeFunction) 时，第一个参数把 KEY 进行回传；
 * 2. 之前与 APP 交互约定的函数方法维持不变；
 *
 * 示例：
 *     getTradeFunction('LOGIN_TIMEOUT_KEY', {"time":"20"})
 *     getTradeFunction('PTJY_MMD_CCCX', {"cxfx":"1", "zqdm": "000001"})
 *     getTradeFunction('PTJY_MMD_LSCJ', {"glbz":"true","zqdm":"132004","qsrq":"20170121","zzrq":"20171223"})
 *     getTradeFunction('GET_DLXX_KEY', {"ywlx":"ptjy"})
 *     getTradeFunction('GET_FIXED_PARAMETER_KEY', {"ywlx":"ptjy"})
 *     getTradeFunction('COMMON_TRADE_PTJY_INTERFACE', {'key': 'ptjy/ptyw/zjcx', 'value': {}})
 */
var TRADE_FUNCTION = function () {
    var CONSTANTS = {
        // 设置交易锁屏的超时时间
        LOGIN_TIMEOUT_KEY: 'LOGIN_TIMEOUT_KEY',
        // 普通交易_买卖点_持仓查询
        PTJY_MMD_CCCX: 'PTJY_MMD_CCCX',
        // 普通交易_买卖点_历史成交
        PTJY_MMD_LSCJ: 'PTJY_MMD_LSCJ',

        // 普通交易_获取股东代码列表
        PTJY_GDDM_KEY: 'PTJY_GDDM_KEY',
        // 普通交易_获取普通交易下所有股东代码列表
        PTJY_GDDM_ALL_KEY: 'PTJY_GDDM_ALL_KEY',
        // 普通交易_获取可买卖数量查询
        PTJY_KMMSLCX_KEY: 'PTJY_KMMSLCX_KEY',
        // 普通交易_获取可买卖数量查询2
        PTJY_KMMSLCX2_KEY: 'PTJY_KMMSLCX2_KEY',
        // 普通交易_委托下单请求
        PTJY_WTXD_KEY: 'PTJY_WTXD_KEY',
        // 普通交易_交易条件委托查询
        PTJY_JYTJWTCX_KEY: 'PTJY_JYTJWTCX_KEY',

        // 获取登录信息
        GET_DLXX_KEY: 'GET_DLXX_KEY',
        // 获取固定入参
        GET_FIXED_PARAMETER_KEY: 'GET_FIXED_PARAMETER_KEY',

        // 设置 SM4 密钥
        KDS_ENCRYPT_KEY: 'KDS_ENCRYPT_KEY',
        // 通用_交易_普通交易_接口
        COMMON_TRADE_PTJY_INTERFACE: 'COMMON_TRADE_PTJY_INTERFACE'
    };

    function getTradeFunction(functionKey, json) {
        switch (functionKey) {
            case CONSTANTS.LOGIN_TIMEOUT_KEY:
                // 设置交易锁屏的超时时间
                setLocalData(functionKey, json.time);
                break;
            case CONSTANTS.PTJY_MMD_CCCX:
                // 普通交易_买卖点_持仓查询
                getMmdCccx(functionKey, json);
                break;
            case CONSTANTS.PTJY_MMD_LSCJ:
                // 普通交易_买卖点_历史成交
                getMmdLscj(functionKey, json);
                break;
            case CONSTANTS.GET_DLXX_KEY:
                // 获取登录信息
                getDlxx(functionKey, json);
                break;
            case CONSTANTS.GET_FIXED_PARAMETER_KEY:
                // 获取固定入参
                getFixedParameter(functionKey, json);
                break;
            case CONSTANTS.KDS_ENCRYPT_KEY:
                // 设置 SM4 密钥
                setLocalData('KDS_ENCRYPT_KEY', JSON.stringify(json));
                break;
            case CONSTANTS.COMMON_TRADE_PTJY_INTERFACE:
                // 通用_交易_普通交易_接口
                commonTradePtjyInterface(functionKey, json);
                break;
            default:
                // 闪电买卖功能模块
                if ((typeof SDMM === 'undefined' ? 'undefined' : _typeof(SDMM)) === 'object') {
                    SDMM.getTradeFunction(functionKey, json);
                }
                break;
        }
    }

    /**
     * 普通交易_买卖点_持仓查询
     */
    function getMmdCccx(functionKey, json) {
        ajaxPost(getApiUrl() + 'ptjy/ptyw/cbcx', json, function (response) {
            callbackTradeFunction(functionKey, JSON.stringify(response));
        }, null, {
            issuccess: false,
            error: function error(XMLHttpRequest, textStatus, errorThrown) {
                var err = getErrMessage(XMLHttpRequest, textStatus, errorThrown);
                callbackTradeFunction(functionKey, JSON.stringify(err));
            }
        });
    }

    /**
     * 普通交易_买卖点_历史成交
     */
    function getMmdLscj(functionKey, json) {
        ajaxPost(getApiUrl() + 'ptjy/ptyw/dsztlsmmdcx', json, function (response) {
            callbackTradeFunction(functionKey, JSON.stringify(response));
        }, null, {
            issuccess: false,
            error: function error(XMLHttpRequest, textStatus, errorThrown) {
                var err = getErrMessage(XMLHttpRequest, textStatus, errorThrown);
                callbackTradeFunction(functionKey, JSON.stringify(err));
            }
        });
    }

    /**
     * 获取登录信息
     */
    function getDlxx(functionKey, json) {
        var user, body, dlxx;
        switch (json.ywlx) {
            case 'ptjy':
                user = JSON.parse(getLocalData('ActivePTJYUser') || '{}');
                body = user.body || {};
                dlxx = body.dlxx || {};
                callbackTradeFunction(functionKey, JSON.stringify(dlxx));
                break;
            case 'rzrq':
                user = JSON.parse(getLocalData('ActiveRZRQUser') || '{}');
                body = user.body || {};
                dlxx = body.dlxx || {};
                callbackTradeFunction(functionKey, JSON.stringify(dlxx));
                break;
        }
    }

    /**
     * 获取固定入参
     */
    function getFixedParameter(functionKey, json) {
        var data, head;
        switch (json.ywlx) {
            case 'ptjy':
                data = JSON.parse(getLocalData('ActivePTJYUser') || '{}');
                head = data.head || {};
                callbackTradeFunction(functionKey, JSON.stringify(head));
                break;
            case 'rzrq':
                data = JSON.parse(getLocalData('ActiveRZRQUser') || '{}');
                head = data.head || {};
                callbackTradeFunction(functionKey, JSON.stringify(head));
                break;
        }
    }

    /**
     * 通用_交易_普通交易_接口
     */
    function commonTradePtjyInterface(functionKey, json) {
        var tradeInterface = json.key;
        ajaxPost(getApiUrl() + tradeInterface, json.value, function (response) {
            callbackAppTradeFunction(functionKey, JSON.stringify(jsonContainer(tradeInterface, response)));
        }, null, {
            issuccess: false,
            error: function error(XMLHttpRequest, textStatus, errorThrown) {
                var err = getErrMessage(XMLHttpRequest, textStatus, errorThrown);
                callbackAppTradeFunction(functionKey, JSON.stringify(jsonContainer(tradeInterface, err)));
            }
        });
    }

    function getErrMessage(xhr, textStatus, errorThrown) {
        var message = '';
        if (xhr.status == 0) {
            if (xhr.statusText == 'timeout') {
                message = '您的网络不给力，请稍后再试。';
            } else {
                message = '网络请求失败，请稍后再试。';
            }
        } else if (xhr.status == 400 || xhr.status == 404) {
            message = '网络连接失败。';
        } else if (xhr.status == 500 || xhr.status == 502) {
            message = '服务器繁忙，请稍后再试。';
        } else {
            message = '网络请求失败，稍后再试。';
        }

        var err = { 'textStatus': textStatus, 'errorThrown': message };
        return err;
    }

    /**
     * JSON 容器，包一层数据
     * @param  {[type]} key  [description]
     * @param  {[type]} json [description]
     * @return {[type]}      [description]
     */
    function jsonContainer(key, json) {
        return {
            key: key,
            value: json
        };
    }

    return {
        CONSTANTS: CONSTANTS,
        getTradeFunction: getTradeFunction
    };
}();