var app = angular.module('jrzxg', []);
app.controller('jrzxgController', function ($rootScope, $scope, $http, $compile) {
    // 设置导航栏的显示
    appXgsgApi.setTitleType("0", "3");
    // “全部添加自选” 是否不可用
    $scope.isDisabled = true;

    jrzxgApi.show($scope);

    /**
     * 加自选
     */
    $scope.addUserStock = function (row) {
        jrzxgApi.addUserStock(row);
        // 如果列表中全部已添加自选，则需要修改 “全部添加自选” 按钮的状态
        $scope.isDisabled = !jrzxgApi.isExistNotAdded($scope);
    };

    /**
     * 全部添加自选
     */
    $scope.addAllUserStock = function () {
        if (!$scope.isDisabled) {
            jrzxgApi.addAllUserStock($scope);
            $scope.isDisabled = true;
        }
    };

    /**
     * 点击展开/折叠卡片内容
     */
    $scope.rowClick = function (row) {
        if (row.show) {
            row.show = false;
        } else {
            row.show = true;
        }
    };
});

/**
 * 加入自选股接口
 */
var jrzxgApi = function (xgsgApi, browser) {

    function show($scope) {
        xgsgApi.showJrzxg($scope, componentWillMount);
    }

    function componentWillMount($scope) {
        console.log("------- jrzxgApi >>>> componentWillMount -------------------");

        $scope["jrzxgData"] = $scope.zqcxData || [];

        // 翻译新股状态
        xgsgApi.translateXGZT($scope["jrzxgData"]);

        //  筛选上市的 （包括今日上市的）
        $scope["jrzxgData"] = screenListed($scope["jrzxgData"]);

        // 隐藏所有行
        allRowShow($scope, false);

        // 添加行数据的自定义属性
        addRowAttr($scope);

        // 查询用户自选股列表
        cheakUserStockInfo($scope);

        $scope.isDisabled = !isExistNotAdded($scope);

        console.log("[显示]" + JSON.stringify($scope["jrzxgData"]));
    }

    /**
     * 筛选上市的 （包括今日上市的）
     */
    function screenListed(array) {
        var listData = [];
        var len = array.length;
        for (var i = 0; i < len; i++) {
            var row = array[i];
            if (row.xgzt == "3" || row.xgzt == "5") {
                listData.push(row);
            }
        }
        return listData;
    }

    // 添加行数据的自定义属性
    function addRowAttr($scope) {
        var len = $scope.jrzxgData.length;
        for (var i = 0; i < len; i++) {
            var row = $scope.jrzxgData[i];
            setRowStockInfo(row, false);
            // 给长数换算添加中文单位
            row["zqslF"] = addChineseUnit(row["zqsl"]);
            if (!isNaN(row.zqje)) {
                row["zqjesmF"] = addChineseUnit(row.zqje);
            }
        }
        if (!isNaN($scope.zjkys)) {
            $scope["zjkyssm"] = (new Number($scope.zjkys) / 10000).toFixed(2);
        }
    }

    /**
     * 设置行自选股信息
     * @param {[type]} row [行数据]
     * @param {[type]} bl  [true: 已添加, false: 未添加]
     */
    function setRowStockInfo(row, bl) {
        // 已添加
        if (bl) {
            row["added"] = "added";
            row["isHaveAdd"] = true;
            row["addBtnText"] = "已添加";
        }
        // 未添加
        else {
                row["added"] = "";
                row["isHaveAdd"] = false;
                row["addBtnText"] = "加自选";
            }
    }

    /**
     * 所有行的显示
     */
    function allRowShow($scope, state) {
        var len = $scope.jrzxgData.length;
        for (var i = 0; i < len; i++) {
            var row = $scope.jrzxgData[i];
            row["show"] = state;
        }
    }

    /**
     * 查询用户自选股列表
     */
    function cheakUserStockInfo($scope) {
        var array = [];
        var len = $scope.jrzxgData.length;
        var obj = {};
        if (len == 0) {
            return;
        }
        for (var i = 0; i < len; i++) {
            var row = $scope.jrzxgData[i];
            obj = {
                "marketId": row.jysdm,
                "stockName": row.zqmc,
                "stockCode": row.gpdm
            };
            array.push(obj);
        }
        array = JSON.stringify(array);

        /**
         * 设置我的自选股列表 【这个方法需要放在调用原生接口前面】
         */
        window["setUserStockList"] = function (array) {
            var len = array.length;
            if (len == 0) {
                return;
            }
            var haveStock = {};
            for (var i = 0; i < len; i++) {
                haveStock[array[i].stockCode] = array[i];
            }
            len = $scope.jrzxgData.length;
            for (var i = 0; i < len; i++) {
                var row = $scope.jrzxgData[i];
                // 已经添加自选股
                if (haveStock[row.gpdm]) {
                    if (browser == 'ios') {
                        setRowStockInfo(row, true);
                    } else {
                        angular.element($("body")).scope().$apply(function () {
                            setRowStockInfo(row, true);
                        });
                    }
                }
            }

            if (browser == 'ios') {
                $scope.isDisabled = !jrzxgApi.isExistNotAdded($scope);
            } else {
                angular.element($("body")).scope().$apply(function () {
                    $scope.isDisabled = !jrzxgApi.isExistNotAdded($scope);
                });
            }

            window["setUserStockList"] = null;
        };

        if (browser == 'android') {
            KDS_Native.cheakUserStockInfo(array);
        } else if (browser == 'ios') {
            loadURL("KDS_Native://cheakUserStockInfo:" + array);
        }
    }

    /**
     * 批量添加自选股
     */
    function batchAddUserStock(array) {
        if (browser == 'android') {
            KDS_Native.batchAddUserStock(array);
        } else if (browser == 'ios') {
            loadURL("KDS_Native://batchAddUserStock:" + array);
        }
        showMessageTips("添加自选股完成");
    }

    /**
     * 添加自选【单】
     */
    function addUserStock(row) {
        var array = [{
            "marketId": row.jysdm,
            "stockName": row.gpmc,
            "stockCode": row.gpdm
        }];
        setRowStockInfo(row, true);
        batchAddUserStock(JSON.stringify(array));
    }

    /**
     * 全部添加自选
     */
    function addAllUserStock($scope) {
        var array = [];
        var len = $scope.jrzxgData.length;
        if (len == 0) {
            return;
        }
        for (var i = 0; i < len; i++) {
            var row = $scope.jrzxgData[i];
            // 已经添加自选股
            if (!row["isHaveAdd"]) {
                array.push({
                    "marketId": row.jysdm,
                    "stockName": row.zqmc,
                    "stockCode": row.gpdm
                });
                setRowStockInfo(row, true);
            }
        }
        batchAddUserStock(JSON.stringify(array));
    }

    /**
     * 是否存在未添加的
     */
    function isExistNotAdded($scope) {
        var len = $scope.jrzxgData.length;
        if (len == 0) {
            return false;
        }
        for (var i = 0; i < len; i++) {
            if (!$scope.jrzxgData[i].isHaveAdd) {
                return true;
            }
        }
        return false;
    }

    return {
        show: show,
        addUserStock: addUserStock,
        addAllUserStock: addAllUserStock,
        isExistNotAdded: isExistNotAdded
    };
}(xgsgApi, params.browser);