/**
 * 闪电买卖
 *
 * SDMM.getTradeFunction('PTJY_GDDM_KEY', {})
 * SDMM.getTradeFunction('PTJY_GDDM_ALL_KEY', {})
 * SDMM.getTradeFunction('PTJY_KMMSLCX_KEY', {})
 * SDMM.getTradeFunction('PTJY_KMMSLCX2_KEY', {})
 * SDMM.getTradeFunction('PTJY_WTXD_KEY', {})
 * SDMM.getTradeFunction('PTJY_JYTJWTCX_KEY', {})
 * SDMM.hideAppWebView()
 * SDMM.completeMethod()
 */
var SDMM = function (window, callbackTradeFunction) {
    var $scope = null;
    var $compile = null;

    function setController(o) {
        $scope = o.$scope;
        $compile = o.$compile;
    }

    function getTradeFunction(functionKey, json) {
        switch (functionKey) {
            // 获取股东代码列表
            case TRADE_FUNCTION.CONSTANTS.PTJY_GDDM_KEY:
                var gddm = getGDDM(json.jysdm);
                callbackTradeFunction(functionKey, JSON.stringify(gddm));
                break;
            // 获取普通交易下所有股东代码列表
            case TRADE_FUNCTION.CONSTANTS.PTJY_GDDM_ALL_KEY:
                var gdlb = getPptjyGdlb();
                callbackTradeFunction(functionKey, JSON.stringify({ gdlb: gdlb }));
                break;
            // 获取可买卖数量查询
            case TRADE_FUNCTION.CONSTANTS.PTJY_KMMSLCX_KEY:
                kmmslcxQuery(functionKey, json);
                break;
            // 获取可买卖数量查询2
            case TRADE_FUNCTION.CONSTANTS.PTJY_KMMSLCX2_KEY:
                kmmslcx2Query(functionKey, json);
                break;
            // 普通交易委托下单请求
            case TRADE_FUNCTION.CONSTANTS.PTJY_WTXD_KEY:
                if (json.mmlb == 'B') {
                    queryIsJysj(functionKey, json, checkHQFXJS);
                } else {
                    queryIsJysj(functionKey, json, showWtxdModal);
                }
                break;
            // 交易条件委托查询
            case TRADE_FUNCTION.CONSTANTS.PTJY_JYTJWTCX_KEY:
                tradeQuery(functionKey, json);
                break;
        }
    }

    /**
     * 获取股东代码
     * @param  {[type]} jysdm [description]
     * @return {[type]}       [description]
     */
    function getGDDM(jysdm) {
        var arr = [];
        try {
            var gdlb = JSON.parse(getLocalData('ActivePTJYUser')).body.gdlb;
            for (var i = 0; i < gdlb.length; i++) {
                if (gdlb[i].jysdm == jysdm) {
                    if (gdlb[i].zcbz === '0') {
                        arr.unshift(gdlb[i]);
                    } else {
                        arr.push(gdlb[i]);
                    }
                }
            }
        } catch (e) {
            arr = [];
        }
        return { gdlb: arr };
    }

    /**
     * 获取股东代码
     * @param  {[type]} marketId [description]
     * @return {[type]}       [description]
     */
    function getPptjyGdlb() {
        var gdlb = [];
        try {
            gdlb = JSON.parse(getLocalData('ActivePTJYUser')).body.gdlb;
        } catch (e) {
            gdlb = [];
        }
        return gdlb;
    }

    /**
     * 进行可买卖数量查询
     * @param  {[type]} functionKey [description]
     * @param  {[type]} postData    [description]
     * @return {[type]}             [description]
     */
    function kmmslcxQuery(functionKey, postData) {
        ajaxPost(getApiUrl() + 'ptjy/ptyw/kmmslcx', postData, function (response) {
            callbackTradeFunction(functionKey, JSON.stringify(response));
        }, null, {
            issuccess: false,
            error: function error(XMLHttpRequest, textStatus, errorThrown) {
                var err = getErrMessage(XMLHttpRequest, textStatus, errorThrown);
                callbackTradeFunction(functionKey, JSON.stringify(err));
            }
        });
    }

    /**
     * 进行可买卖数量查询2
     * @param  {[type]} functionKey [description]
     * @param  {[type]} postData    [description]
     * @return {[type]}             [description]
     */
    function kmmslcx2Query(functionKey, postData) {
        ajaxPost(getApiUrl() + 'ptjy/ptyw/kmmslcx2', postData, function (response) {
            callbackTradeFunction(functionKey, JSON.stringify(response));
        }, null, {
            issuccess: false,
            error: function error(XMLHttpRequest, textStatus, errorThrown) {
                var err = getErrMessage(XMLHttpRequest, textStatus, errorThrown);
                callbackTradeFunction(functionKey, JSON.stringify(err));
            }
        });
    }

    /**
     * 获取是否是交易时间（1-是；0-否）
     */
    function queryIsJysj(functionKey, json, callback) {
        var successCallback = function successCallback(response) {
            try {
                var fhxx = response.fhlb[0];
                // 是否交易时间（1-是；0-否）
                if (fhxx.sfjysj == '0') {
                    var jysjObj = {
                        isJysj: false, // 是否是交易时间
                        yswtTs: fhxx.fjysjts //非交易时间，夜市委托的提示话术
                    };
                    callback(functionKey, json, jysjObj);
                } else {
                    callback(functionKey, json);
                }
            } catch (erro) {
                callback(functionKey, json);
            }
        };
        var errorCallback = function errorCallback(response) {
            callback(functionKey, json);
        };
        var options = {
            iserror: false,
            error: function error(xhr, status, _error) {
                callback(functionKey, json);
            }
        };

        ajaxPost(getApiUrl() + 'ptjy/ptyw/sfjysj', {}, successCallback, errorCallback, options);
    }

    /**
     * 校验是否需要退市风险警示
     * jysjObj[交易时间的相关信息]
     */
    function checkHQFXJS(functionKey, postData, jysjObj) {
        var request = {
            zqdm: postData.zqdm,
            jysdm: postData.jysdm
        };
        ajaxPost(getApiUrl() + 'ptjy/zhyw/hqfxjs', request, function (response) {
            try {
                var sfts = response.fhxx[0].sfts;
                // 需要退市风险警示
                if (sfts == '1') {
                    var modal = {};
                    modal['title'] = '风险提示';
                    modal['msg'] = [{ 'value': response.fhxx[0].fxxx }];
                    Dialog.alert($scope, $compile, modal, {
                        leftButtonClick: function leftButtonClick() {
                            hideAppWebView();
                            Dialog.closeAlert('modalTips');
                        },
                        rightButtonClick: function rightButtonClick() {
                            Dialog.closeAlert('modalTips');
                            showWtxdModal(functionKey, postData, jysjObj);
                        }
                    });
                } else {
                    // 不需要提示，进行下一步流程
                    // 显示委托下单弹出框
                    showWtxdModal(functionKey, postData, jysjObj);
                }
            } catch (erro) {
                console.log('ptjy/zhyw/hqfxjs 接口返回的数据格式不对！');
                hideAppWebView();
            }
        }, null, {
            completeMethod: completeMethod,
            isloaddingrun: false
        });
    }

    /**
     * 显示买入/卖出确认框
     * @param  {[type]} functionKey [description]
     * @param  {[type]} postData    [description]
     * @param  {[type]} jysjObj     [交易时间的相关信息]
     * @return {[type]}             [description]
     */
    function showWtxdModal(functionKey, postData, jysjObj) {
        var modal = {};
        if (postData.mmlb == 'B') {
            modal['title'] = '买入确认';
        } else if (postData.mmlb == 'S') {
            modal['title'] = '卖出确认';
        } else {
            return;
        }
        var wtlxText = '限价委托';
        if (postData.wtlx === '30') {
            wtlxText = '盘后固定价格';
        }
        modal['content'] = [{ 'key': '委托方式', 'value': wtlxText }, { 'key': '股东代码', 'value': postData.gddm }, { 'key': '证券代码', 'value': postData.zqdm }, { 'key': '证券名称', 'value': postData.zqmc }, { 'key': '委托价格', 'value': postData.wtjg }, { 'key': '委托数量', 'value': postData.wtsl }];
        // 非交易时间，夜市委托的时候，弹框中显示提示文字
        if (jysjObj && !jysjObj.isJysj) {
            modal.btnBeforeDynamic = { type: 'cageTipsAndYswt', yswtts: jysjObj.yswtTs };
        } else {
            modal.btnBeforeDynamic = { type: 'cageTips' };
        }
        var bl = true;
        Dialog.alert($scope, $compile, modal, {
            rightIcoHide: true,
            isBodyScroll: true,
            leftButtonClick: function leftButtonClick() {
                hideAppWebView();
                Dialog.closeAlert('modalTips');
            },
            rightButtonClick: function rightButtonClick() {
                Dialog.closeAlert('modalTips');
                if (bl) {
                    bl = false;
                    wtxd(functionKey, postData);
                }
            }
        });
    }

    /**
     * 委托下单
     * @param  {[type]} functionKey [description]
     * @param  {[type]} postData    [description]
     * @return {[type]}             [description]
     */
    function wtxd(functionKey, postData) {
        var options = {
            completeMethod: completeMethod
        };
        if (postData.mmlb === 'B') {
            options.specifyServerErrorCode = PROPER_RECONSTRUCTION_FLOOR_TRADING.SPECIFY_SERVER_ERROR_CODE;
            options.specifyServerErrorCodeMethod = PROPER_RECONSTRUCTION_FLOOR_TRADING.start;
        }

        // 普通交易闪电买卖引用 CDR 模块
        CDR.init(options, hideAppWebView.bind(this, null));
        // 普通交易闪电买卖引用 沪伦通 模块
        HLT.init(options, hideAppWebView.bind(this, null));
        // 普通交易引用 科创板 模块
        KCB.init(options, hideAppWebView.bind(this, null));

        ajaxPost(getApiUrl() + 'ptjy/ptyw/wtxd', postData, function (response) {
            Dialog.successAlert('委托提交成功,流水号：' + response.htxx[0].wtxh, '提示');
            Dialog.confirm = function () {
                confirm();
                callbackTradeFunction(functionKey, JSON.stringify(response));
            };
        }, function (response) {
            Dialog.errorAlert('委托提交失败：' + response.cljg[0].message, '提示');
        }, options);
    }

    /**
     * 交易查询
     * @param  {[type]} functionKey [description]
     * @param  {[type]} postData    [description]
     * @return {[type]}             [description]
     */
    function tradeQuery(functionKey, postData) {
        ajaxPost(getApiUrl() + 'ptjy/ptyw/ddxxcx', postData, function (response) {
            callbackTradeFunction(functionKey, JSON.stringify(response));
        }, null, {
            issuccess: false,
            error: function error(XMLHttpRequest, textStatus, errorThrown) {
                var err = getErrMessage(XMLHttpRequest, textStatus, errorThrown);
                callbackTradeFunction(functionKey, JSON.stringify(err));
            }
        });
    }

    function hideAppWebView(clossAppAlert) {
        var json = { 'show': '0' };
        if (clossAppAlert) {
            json.clossAppAlert = 'true';
        }
        callbackTradeFunction('APP_WEB_VIEW_KEY', JSON.stringify(json));
    }

    function confirm() {
        hideAppWebView();
        Dialog.closeAlert('CustomAlert');
    }

    function completeMethod(xhr, textStatus, callback) {
        if (textStatus === 'success') {
            if (xhr.responseText.indexOf('MP1B000000') === -1) {
                Dialog.confirm = function () {
                    confirm();
                };
            }
        } else {
            Dialog.confirm = function () {
                confirm();
            };
        }
        if (callback != null) {
            callback();
        }
    }

    function getErrMessage(xhr, textStatus, errorThrown) {
        var message = '';
        if (xhr.status == 0) {
            if (xhr.statusText == 'timeout') {
                message = '您的网络不给力，请稍后再试。';
            } else {
                message = '网络请求失败，请稍后再试。';
            }
        } else if (xhr.status == 400 || xhr.status == 404) {
            message = '网络连接失败。';
        } else if (xhr.status == 500 || xhr.status == 502) {
            message = '服务器繁忙，请稍后再试。';
        } else {
            message = '网络请求失败，稍后再试。';
        }

        var err = { 'textStatus': textStatus, 'errorThrown': message };
        return err;
    }

    /**
     * 重写注销方法
     */
    window.Exit = function (callback) {
        return function () {
            ClearLoginInfo();
            // 超时登录框存在则删除
            if ($("#TimeOut").length > 0) {
                $("#TimeOut").remove();
            }
            hideAppWebView(true);
        };
    }(window.Exit);

    return {
        getTradeFunction: getTradeFunction,
        setController: setController,
        hideAppWebView: hideAppWebView,
        completeMethod: completeMethod
    };
}(window, callbackTradeFunction);

var app = angular.module('sdmm', []);
app.directive('ytStringHtml', function () {
    return function (scope, el, attr) {
        if (attr.ytStringHtml) {
            scope.$watch(attr.ytStringHtml, function (html) {
                el.html(html || '');
            });
        }
    };
});
app.controller('sdmmController', function ($rootScope, $scope, $http, $compile) {
    SDMM.setController({ $scope: $scope, $compile: $compile });
    // 兼容弹框点击事件
    getControllerMethod($rootScope, $scope, $http, $compile);
});